from setuptools import setup, find_packages

setup(
    name='wopexamplecar',
    version='1.0',
    description='Simple car example for experimenting Wopmars',
    author='Luc Giffon - TAGC',
    author_email='luc.giffon@gmail.com',
    packages=find_packages(),
)
