import os
from setuptools import setup
from setuptools import find_packages

setup(
    name='wopexamplesnp',
    version='1.0',
    description='SNP example for experimenting Wopmars',
    author='Aitor Gonzalez - TAGC',
    author_email='aitor.gonzalez@univ-amu.fr',
    packages=find_packages(),
    )

